



//  LCD Master Clock using Udo Klein's DCF77 library
//  http://www.brettoliver.org.uk
//
//  Copyright 2014 Brett Oliver
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program. If not, see http://www.gnu.org/licenses/
//
//  Summer advance and Winter Retards code thanks to Peter Hillyer
//
// Based on the DCF77 library by Udo Klein
// http://blog.blinkenlight.net/experiments/dcf77/dcf77-library/
// http://blog.blinkenlight.net/experiments/dcf77/simple-clock/
// v 3 LedControl removed and Pin 13 used for dcf77 monitor as spare
// v 4 LED30 removed as conflicting with DCF77 input pin  pin2
// v 5 stripped data from loop
// v6 int and pinmode values  added Quartz motor driver and version number display added
// v7 add qtr chime
// v8 added hour chime
// v9 added flash 1 sec pulse pin
// v10 added void statements
// v11 added blink without delay
// v12 as above
// v13 added extra count and leap second
// v14 added accuracy stats
// v15 added seconds miss and slow pulse
// v16 date and time of missed pulse displayed
// v17 do not use
// v18 clock state added to row 02
// v19 leading zeroes added to date on miss & extra pulse
// v20 infrared switch added to pin 13
// v22 remove 15 sec pulse change DCF77 monitor pin to use this op instead
// v23 modified hour chime code
// v24 Qtr chime modified to stop it repeating 30 seconds later
// v25 Modified to run on Udo Klein's v3 DCF77 library
// v26 Auto Summer/Winter correction added
// v27 Summer Winter correction switch added pin 13/leading zeroes added to signal match/added leading zeroes to bstCount
// v28 Accuracy added back in and namespace errors corrected
//#include <LedControl.h>
#include <Wire.h>
#include <dcf77.h>
#include <LiquidCrystal_I2C.h>
using namespace Internal;



//unsigned long delaytime=250;
//**********************
// set the LCD address to 0x27 for a 20 chars 4 line display
// Set the pins on the I2C chip used for LCD connections:
//                    addr, en,rw,rs,d4,d5,d6,d7,bl,blpol
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address

#define NOP __asm__ __volatile__ ("nop\n\t")  //define NOP instruction
//**********************

const uint8_t dcf77_analog_sample_pin = 5; // not used
const uint8_t dcf77_sample_pin = 2;  // DCF77 input signal
const uint8_t dcf77_inverted_samples = 0;
const uint8_t dcf77_analog_samples = 0;
//const uint8_t dcf77_monitor_pin = 3;  
const uint8_t dcf77_monitor_led = 3; // v3
uint8_t ledpin(const uint8_t led){ //v3
  return led; //v3
} //v3

const int8_t timezone_offset = -1;  // GB is one hour behind CET/CEST




int infraredSwval = 0; // infrared value
int infraredSw = 13; //


int extracount = 0; // where quartz seconds needs to miss a pulse
int hourextra = 00; // hour last miss pulse variable
int dayextra = 00; // day last miss pulse variable
int monthextra = 00; // month last miss pulse variable
int minuteextra = 00; // minute last miss pulse variable
int secondextra = 00; // second last miss pulse variable
int yearextra = 00;
int signalQual = 0; // computed once per minute and indicates how well 
// the received signal matches the locally synthesized reference signal max 50

int secondsnow = 0; // previous second
int yearmiss = 00;
int daymiss = 00; // day last extra pulse variable
int monthmiss = 00; // month last extra pulse variable
int secsmiss = 0; // works out is seconds need to an extra pulse or miss a pulse

int summertest = 0; // equals 1 for summertime and 0 for wintertime

int misscount = 0; // ensures miscount goes to 0 on startup
int hourmiss = 00; // hour last extra pulse variable
int minutemiss = 00; // minute last extra pulse variable
int secondmiss = 00; // second last extra pulse variable
int hoursync = 00; // hour last sync variable
int minutesync = 00; // minute last sync variable
int secondsync = 00; // second last sync variable
//int ledStateOdd = HIGH;             // ledState used to set the 1 second LED initial state
//int ledStateEven = HIGH;              // ledState used to set the 2nd 1 second LED initial state
int quartzmotor1 = HIGH;              // ledState used to set the quartz motor pin A2 initial state
int quartzmotor2 = LOW;              //  ledState used to set the quartz motor pin A3 initial state

int led = 7; // min pulse
int led30 = 8; // 30 sec pulse
int led05 = 11; // 05 sec pulse
int led10 = 10; // 10 sec pulse
//int led15 = 3; // 15 sec pulse
int led20 = 9; // 20 sec pulse
const int led01 = 12; // 1 sec pulse
int led01state = LOW; // ledState used to set led01
//long previousMillis = 0; //// will store last time led01 was updated
//long interval = 500;           // interval at which to blink (milliseconds)
int ledh = 6; // 1 hour pulse
int ledd = 5; // 1 day pulse
int ledQchime = 4; // qtr hour chime pulse
int trigQchime = A0; // qtr hour chime trig pulse
int ledchime =A1;  //  hour chime pulse
int quartz01 =A3;  //  Quartz clock motor pulse 01
int quartz02 =A2;  //  Quartz clock motor pulse 02
int monthval = 0;
int dayval = 0;
int hours = 0;
int minutes = 0;
int seconds = 0;
int years = 0;
int months = 0;
int days = 0;
int sumwinSwitch  = 3; // HIGH to advance/retard clock depending om GMT+1 or 0

// Auto Summer winter correction
int bstCount          = 3600; // count for bst advance\retard (3600 seconds need to be added or stopped from the clock motor)
boolean retard        = false;  // true if clock going back
boolean advance       = false;  // true if clock going forward


boolean lastBst       = false;  // BST just changed flag
boolean oneSecTick    = false;   // changes state once a second (end of <loop?)
boolean coilStatus    = false;  // clock coil status
boolean clockRun      = true;   // when false clock motor will stop used for retard and sync 
boolean firstRun      = true;   // set to false after first pass of main loop
byte dateDowCount     = 0;      // lcd date day of week counter
byte bst              = 0;      // equals 0 summertime or 2 for wintertime

//********************

namespace Timezone {
  uint8_t days_per_month(const Clock::time_t &now) {
    switch (now.month.val) {
    case 0x02:
      // valid till 31.12.2399
      // notice year mod 4 == year & 0x03
     return 28 + ((now.year.val != 0) && ((bcd_to_int(now.year) & 0x03) == 0)? 1: 0);
            case 0x01: case 0x03: case 0x05: case 0x07: case 0x08: case 0x10: case 0x12: return 31;
            case 0x04: case 0x06: case 0x09: case 0x11:                                  return 30;
            default: return 0;
    }
  }   

  void adjust(Clock::time_t &time, const int8_t offset) {
    // attention: maximum supported offset is +/- 23h

    int8_t hour = BCD::bcd_to_int(time.hour) + offset;

    if (hour > 23) {
      hour -= 24;
      uint8_t day = BCD::bcd_to_int(time.day) + 1;
      uint8_t weekday = BCD::bcd_to_int(time.weekday) + 1; //v3
      if (day > days_per_month(time)) {
        day = 1;
        uint8_t month = BCD::bcd_to_int(time.month);
        ++month;
        if (month > 12) {
          month = 1;
          uint8_t year = BCD::bcd_to_int(time.year);                  
          ++year;
          if (year > 99) {
            year = 0;
          }
          time.year = BCD::int_to_bcd(year);
        }                
        time.month = BCD::int_to_bcd(month);
      }
      time.day = BCD::int_to_bcd(day);
      time.weekday = BCD::int_to_bcd(weekday); //v3
    }

    if (hour < 0) {
      hour += 24;
      uint8_t day = BCD::bcd_to_int(time.day) - 1;
      uint8_t weekday = BCD::bcd_to_int(time.weekday) - 1; //v3
      if (day < 1) {
        uint8_t month = BCD::bcd_to_int(time.month);
        --month;
        if (month < 1) {
          month = 12;
          int8_t year = BCD::bcd_to_int(time.year);                  
          --year;
          if (year < 0) {
            year = 99;
          }
          time.year = BCD::int_to_bcd(year);
        }                
        time.month = BCD::int_to_bcd(month);
        day = days_per_month(time);
      }
      time.day = BCD::int_to_bcd(day);
      time.weekday = BCD::int_to_bcd(weekday); //v3
    }

    time.hour = BCD::int_to_bcd(hour);
  }
}

uint8_t sample_input_pin() {
  const uint8_t sampled_data =
    dcf77_inverted_samples ^ (dcf77_analog_samples? (analogRead(dcf77_analog_sample_pin) > 200)
    : digitalRead(dcf77_sample_pin));

  //digitalWrite(dcf77_monitor_pin, sampled_data); //removed v3
  digitalWrite(ledpin(dcf77_monitor_led), sampled_data); //v3
  return sampled_data;
}

void setup() {
  digitalWrite(led30, LOW); // turn 30 sec clocks off at start
  digitalWrite(ledQchime, HIGH); // turn Qtr chime off at start
  digitalWrite(ledchime, LOW); // turn Hour chime off at start
  digitalWrite(trigQchime, HIGH); // turn Hour chime off at start
 

  lcd.begin(20,4);   // initialize the lcd for 20 chars 4 lines, turn on backlight 
  //using namespace DCF77_Encoder; //v25
  //*************
 // lcd.backlight(); // backlight on not needed as controlled by 7 MAX2719
  lcd.setCursor(0,0); //Start at character 0 on line 0
  lcd.print("DCF77 Master Clk v28");  





  //***************
  Serial.begin(9600);
  /*  Serial.println();
   Serial.println(F(" DCF77 Master Clock "));
   Serial.println(F("(c) Brett Oliver 2014"));
   Serial.println(F("http://www.brettoliver.org.uk"));
   Serial.println(F("Based on the DCF77 library by Udo Klein"));
   Serial.println(F("www.blinkenlight.net"));
   Serial.println();
   Serial.print(F("Sample Pin:     ")); 
   Serial.println(dcf77_sample_pin);
   Serial.print(F("Inverted Mode:  ")); 
   Serial.println(dcf77_inverted_samples);
   Serial.print(F("Analog Mode:    ")); 
   Serial.println(dcf77_analog_samples);
   Serial.print(F("Monitor Pin:    ")); 
   Serial.println(dcf77_monitor_pin);
   Serial.print(F("Timezone Offset:")); 
   Serial.println(timezone_offset);
   Serial.println();
   Serial.println();
   Serial.println(F("Initializing..."));
   */
 // pinMode(dcf77_monitor_pin, OUTPUT);//removed v3

  pinMode(dcf77_sample_pin, INPUT);
  digitalWrite(dcf77_sample_pin, HIGH);
pinMode(ledpin(dcf77_monitor_led), OUTPUT); //v3

  //*******************
  pinMode(led, OUTPUT);
  pinMode(led30, OUTPUT);
  pinMode(led05, OUTPUT);
  pinMode(led10, OUTPUT);
 // pinMode(led15, OUTPUT);
  pinMode(led20, OUTPUT);
  pinMode(led01, OUTPUT);
  pinMode(ledh, OUTPUT);
  pinMode(ledd, OUTPUT);
  pinMode(ledQchime, OUTPUT);
  pinMode(trigQchime, OUTPUT);
  pinMode(ledchime, OUTPUT);
  pinMode(quartz01, OUTPUT);
  pinMode(quartz02, OUTPUT);
 
  pinMode(infraredSw, INPUT); // infrared sensor output
  pinMode(sumwinSwitch, INPUT); // sumwinSwitch input pin
  

  //*******************

  DCF77_Clock::setup();
  DCF77_Clock::set_input_provider(sample_input_pin);


  // Wait till clock is synced, depending on the signal quality this may take
  // rather long. About 5 minutes with a good signal, 30 minutes or longer
  // with a bad signal
  for (uint8_t state = Clock::useless;
         state == Clock::useless || state == Clock::dirty;
         state = DCF77_Clock::get_clock_state()) {

    // wait for next sec
    Clock::time_t now;
    DCF77_Clock::get_current_time(now);

    // render one dot per second while initializing
    static uint8_t count = 0;
    Serial.print('.');
    ++count;
    if (count == 60) {
      count = 0;
      Serial.println();
    }
  }
}

void paddedPrint(BCD::bcd_t n) {
  Serial.print(n.digit.hi);
  Serial.print(n.digit.lo);
}
void LCDpaddedPrint(BCD::bcd_t n) { 
  lcd.print(n.digit.hi); 
  lcd.print(n.digit.lo); 
}



void loop() {

  Clock::time_t now;

  DCF77_Clock::get_current_time(now);
  Timezone::adjust(now, timezone_offset);

  if (now.month.val > 0) {


    //***********
    // get month & day values
    dayval = now.day.val, DEC;
    monthval = now.month.val, DEC;
    // Serial.print(" day ");
    // Serial.print(now.day.val, DEC);
    //  Serial.print(" month ");
    //  Serial.print(now.month.val, DEC);
    //   Serial.print(' ');
    // get month & day values
    // Quartz clock driver
    // toggle Quartz drive 7 & 8 evey second
  }  
    
    lcd.setCursor(13,2);
    //lcd.print(" Wait  ");
    switch (DCF77_Clock::get_clock_state()) {
      // case DCF77::useless: Serial.print(F("useless ")); break;
      case Clock::useless: 
      lcd.print(F(" Fail  ")); 
      break;
      // case DCF77::dirty:   Serial.print(F("dirty: ")); break;
      //  case DCF77::synced:  Serial.print(F("synced: ")); break;
      // case DCF77::locked:  Serial.print(F("locked: ")); break;
      case Clock::dirty:   
      lcd.print(F(" Dirty ")); 
      break;
      case Clock::synced:  
      lcd.print(F(" Sync'd")); 
      break;
      case Clock::locked:  
      lcd.print(F(" Locked")); 
      break;
    }

    //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    /*
    Serial.print("Quartz Frequency");
    Serial.println(16000000L - DCF77_1_Khz_Generator::read_adjustment());
    
    Serial.print("Quartz Accuracy");
    Serial.println(DCF77_Frequency_Control::get_confirmed_precision());
    */
    // Get hours minutes and seconds variables
    hours = BCD::bcd_to_int(now.hour);


    minutes = BCD::bcd_to_int(now.minute);


    seconds = BCD::bcd_to_int(now.second);

    years = BCD::bcd_to_int(now.year);

    months = BCD::bcd_to_int(now.month);

    days = BCD::bcd_to_int(now.day);
    
    Serial.print(hours);
    Serial.print(":");
    Serial.print(minutes);
    Serial.print(":");
    Serial.print(seconds);
   
    
    Serial.print(" ");
    Serial.print(days);
    Serial.print(":");
    Serial.print(months);
    Serial.print(":");
    Serial.println(years);
    


    //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

    // Start of miss pulse detect

   
 
   secsmiss = seconds - secondsnow;

    if (secsmiss ==-59 || secsmiss ==-60 && seconds ==0) // takes account of seconds rollover -59 or leap second -60
    {
      secsmiss = 1;
    } 


    if (secsmiss >=1 && seconds !=60 && !retard) // if zero or less seconds pulse need to be missed. Leap second missed if seconds = 60
    {
      secondsmotor (); // function steps quartz motor
        if(advance) extraPulse(); //if clocks have gone forward pulse clock again (3600 times)
  }
  
   // }

    /*
    if (secsmiss >1) // if extra second pulse required
     {
     delay(100); // add delay so seconds mtor can operate again in the same second
     secondsmotor (); // function steps quartz motor and adds extra pulse if secmiss is >1
     }
     
     */



    if (secsmiss < 1 || seconds == 60) //records time of extra sec second (quart motor needs to loose a second)

    {
      extracount = extracount + 1; //increment extra count total (1sec needs to miss pulse)
      hourextra = hours;
      minuteextra = minutes;
      secondextra = seconds;
      yearextra = years;
      monthextra = months;
      dayextra = days; 
     
    }


    if (secsmiss > 1) //records time of miss second (quart motor needs to add a second)
    {
      misscount = misscount + 1; //increment Miss count total (1sec has missed extra pulse)
       if(advance || retard){        //if in advance or retard add or remove pulses
      if(advance) bstCount++;     //advance: add pulse
      else bstCount--;            //retard: remove pulse
    }
    else extraPulse();            //not in adv or retard pulse clock motor again
      
      hourmiss = hours;
      minutemiss = minutes;
      secondmiss = seconds;
      yearmiss = years;
      monthmiss = months;
      daymiss = days;

    } 



    secondsnow = seconds; 



    if (misscount >99) // resets miss second counter to 0 after 99 pulses
    {
      misscount = 0;
      
    }
    if (extracount >99) // resets miss second counter to 0 after 99 pulses
    {
      extracount = 0;
      
    }


    // }
    // Enable below to analize missed pulses on serial monitor
    //Serial.print(" ");
    // Serial.print("secsmiss ");
    //  Serial.println(secsmiss);

/*
    Serial.print("Slow Seconds ");
    Serial.print(misscount);
    Serial.print(" ");
    Serial.print(hourmiss);
    Serial.print(":");
    Serial.print(minutemiss);
    Serial.print(":");
    Serial.print(secondmiss);
    Serial.print(" ");
    Serial.print(daymiss);
    Serial.print("/");
    Serial.print(monthmiss);
    Serial.print("/");
    Serial.println(yearmiss);


    Serial.print("Fast Seconds ");
    Serial.print(extracount);
    Serial.print(" ");
    Serial.print(hourextra);
    Serial.print(":");
    Serial.print(minuteextra);
    Serial.print(":");
    Serial.print(secondextra);
    Serial.print(" ");
    Serial.print(dayextra);
    Serial.print("/");
    Serial.print(monthextra);
    Serial.print("/");
    Serial.println(yearextra);
*/
    
    // End of miss pulse detect
    
    
   
    
    
    
    // Display time and date on top row of LCD display

    lcd.setCursor(0,0);

    LCDpaddedPrint(now.hour);
    lcd.print(":");
    LCDpaddedPrint(now.minute);
    lcd.print(":");
    LCDpaddedPrint(now.second);
    lcd.print("  ");

    LCDpaddedPrint(now.day);
    lcd.print("/");
    LCDpaddedPrint(now.month);
    lcd.print("/");
    lcd.print("20");
    LCDpaddedPrint(now.year);
    
    // End Display time and date on top row of LCD display
    


/*
    // Star Summer/Winter display

    
   // const int8_t offset_to_utc = timezone_offset + (now.uses_summertime? 2: 1);

    summertest = (abs(offset_to_utc)); // equals 1 if summertime and 2 if wintertime 

    if (summertest ==2) // if wintertime make summertest =0
    {
      summertest = 0;
    }

    //**************
    lcd.setCursor(15,3);
    lcd.print("GMT+");
    // UTCcheck= offset_to_utc;
    // lcd.print(UTCcheck);
    lcd.print(summertest);
    
    // End Summer/Winter display
*/

// DISABLE BST SECTION BELOW TO COMPILE IN DEBUG MODE
  ///*
  // Compute and display BST information
  const int8_t offset_to_utc = timezone_offset + (now.uses_summertime? 2: 1);

  // enable to 2 lines below to activate 
  bst = (abs(offset_to_utc)); // if summertime bst will be 1 
  if (bst == 2) bst = 0; // if wintertime bst will be 2 so change it to 0
 
   //**************
    lcd.setCursor(15,3);
    lcd.print("GMT+");
    // UTCcheck= offset_to_utc;
    // lcd.print(UTCcheck);
    lcd.print(bst);
    
  
  // the line below is enabled for debug use
  //changeBST();    //this is a test switch only for debug use
  if(retard == false && advance == false)
  {    
  lcd.setCursor(0,2);
      lcd.print("Adv/Rtd Off ");
  }
  
  if(firstRun){             //if first pass of program loop ensure lastBST value = current value first computed
    lastBst = bst;
  }
  
  
  else {
    if ((digitalRead(sumwinSwitch) == HIGH && bst == 0) || (retard == 1 && bstCount > 0)) {   //summer to winter clock goes back (misses 3600 pulse)
 
      retard = true;
      bstCount--; 
    }
  
  else if ((digitalRead(sumwinSwitch) == HIGH && bst == 1) || (advance == 1 && bstCount > 0)) {   //winter to summer clock goes forward (add 3600 pulses)
     
      advance = true;
      bstCount--;
    }
    else {
      bstCount = 3600;
      retard = false;
      advance = false;
      
    }
    lastBst = bst;
  }
  
  
  
  
  
 /*
  else {
   if ((lastBst == 1 && bst == 0) || (retard == 1 && bstCount > 0)) {   //summer to winter clock goes back (misses 3600 pulse)
  
      retard = true;
      bstCount--; 
    }
  
  else if ((lastBst == 0 && bst == 1) || (advance == 1 && bstCount > 0)) {   //winter to summer clock goes forward (add 3600 pulses)
     
      advance = true;
      bstCount--;
    }
    else {
      bstCount = 3600;
      retard = false;
      advance = false;
      
    }
    lastBst = bst;
  }

*/
  if(retard || advance){
   
   if(bstCount == 3599){                           // if the count is 3599 process just started
   lcd.setCursor(0,2);
    
     if(retard) lcd.print(" Retard:");
      else lcd.print("Advance:");
    }
   lcd.setCursor(8,2);
 
   if(bstCount < 1000) lcd.print("0");
   else if(signalQual > 9 && bstCount < 100 ) lcd.print("  ");
   else if(bstCount < 10) lcd.print("   ");
    lcd.print(bstCount);
  } 
 // else  displayMultiFunction();                   // display normal multifunction strip
  
  //*/







 // Infrared Detect
 int infraredSw = digitalRead(13); // reads infrared detector output
   // Serial.print ("Infrared ");
   // Serial.println (infraredSw);
   // Serial.println ("");
    if (infraredSw == 1)
    {
      lcd.backlight(); // backlight on
      lcd.display(); //display on  
    } 
      
      else if (infraredSw == 0)
    {  
      lcd.noBacklight(); // LCD backlight off
      lcd.noDisplay(); //LCD display off 
    }
    
 // End Infrared Detect




 // Quarter Chime

  if ((minutes == 15 || minutes == 30 || minutes == 45) && seconds == 0)
  {
    digitalWrite(ledQchime, LOW);
   //digitalWrite(trigQchime, LOW);
  }

  else if
    (minutes == 59 && seconds == 57) 
  {
    digitalWrite(ledQchime, LOW);
  // digitalWrite(trigQchime, LOW);
  }

  else
    digitalWrite(ledQchime, HIGH);
  //  digitalWrite(trigQchime, HIGH);
    
  if ((minutes == 15 || minutes == 30 || minutes == 45) && seconds == 0)
  {
   // digitalWrite(ledQchime, LOW);
   digitalWrite(trigQchime, LOW);
  }

  else if
    (minutes == 59 && seconds == 57) 
  {
   // digitalWrite(ledQchime, LOW);
   digitalWrite(trigQchime, LOW);
  }

  else
   // digitalWrite(ledQchime, HIGH);
    digitalWrite(trigQchime, HIGH);
    
   
  //End of Quarter Chime
  
  // start hour Chime
  // map hours to range 1-12
const uint8_t normalized_hours = hours > 12 ? hours-12 : hours > 0 ? hours : 12;
// compute chime instead of matching it
digitalWrite(ledchime,minutes == 0 && (seconds & 1) == 0 && seconds< 2 * normalized_hours);
  // end hour Chime
  
 
  // Flash 1 sec LED
  // digitalWrite(led01, HIGH);   // turn the LED on (HIGH is the voltage level)
 // delay(25);
//  digitalWrite(led01, LOW);   // turn the LED off (LOW is the voltage level) 

// check to see if it's time to blink the LED; that is, if the 
  // difference between the current time and last time you blinked 
  // the LED is bigger than the interval at which you want to 
  // blink the LED.

 
    

    // if the LED is off turn it on and vice-versa:
    if (led01state == LOW)
    {
      led01state = HIGH;
    }
    else
      led01state = LOW;

    // set the LED with the ledState of the variable:
    digitalWrite(led01, led01state);
    
  
  // End Flash 1 sec LED
  
  
  printPTen(seconds); // prints secs ie 0 every 60 secs
  printP30(seconds); //  prints 0 or 30  on 0 or 30 secs
  printP05(seconds); //  prints every 5 secs
  printP10(seconds); //  prints every 10 secs
 // printP15(seconds); //  prints every 15 secs
  printP20(seconds); //  prints every 20 secs
  printPhour(minutes); // print every hour eg when ever mins at 00
  printPday(hours); // print when hour is 0 eg every 24 hour
  
  
 // Quality display on LCD


  if (seconds >= 0 && seconds <= 10)
  { 
    signalmatch(); // Quality factor
  }

  else if (seconds == 11)
  {
    blankrow3(); //Blanks row 3
  }

else if (seconds == 12)
  {
    lcd.setCursor(0,3);
  lcd.print("Quartz         ");
  }
  else if (seconds >= 13 && seconds <= 23)
  {
    precision(); //quartz confirmed and target precision
  }

  else if (seconds == 24)
  {
    blankrow3(); //Blanks row 3
  }

 else if (seconds >=25 && seconds <= 35 )
  {
      lcd.setCursor(0,3);
      
  signalmatch(); // Quality factor
  }

  else if (seconds == 36)
  {
    blankrow3(); //Blanks row 3
  }
 
 


else if (seconds == 37)
  {
    lcd.setCursor(0,3);
  lcd.print("Quartz         ");
  }
  else if (seconds >= 38 && seconds <= 48)
  {
    precision(); //quartz confirmed and target precision
  }

  else if (seconds == 49)
  {
    blankrow3(); //Blanks row 3
  }

 else if (seconds >=50 && seconds <= 59 )
  {
      lcd.setCursor(0,3);
      
  signalmatch(); // Quality factor
  }

 
  // End of Quality display on LCD
  //$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

  
  
  
  
  
  // Display of Title and display of slow and fast pulses on row 01

/*
  if (retardcount > 0 || retardcount == 119 ) //Prints correction pulse number
  {
    lcd.setCursor(0,1); 
    lcd.print("Winter Retard  ");
    lcd.setCursor(15,1);



    retard(retardcount); // add leading 0 <99
    retardtens(retardcount); // add another leading 0 less than 10
    lcd.print(retardcount);
  }
  else if (retardcount == 120 )  //Prints blank when correction pulse is on last number
  {
    lcd.setCursor(0,1); 
    lcd.print("                    ");
    // lcd.print(retardcount);
  } 
*/

  if (seconds >= 0 && seconds <= 5) // Ignored while clock is correcting forward or retarding
  {

    lcd.setCursor(0,1); //Start at character 0 on line 0
    lcd.print(" DCF77 Master Clock ");      

  }
  
  else if(seconds > 05 && seconds <= 10) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0
    //lcd.print(" DCF77 Master Clock ");  
    lcd.print(" Brett Oliver  v28 ");
  }

  else if(seconds == 11) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0

    lcd.print("                    ");
  }

  else if(seconds > 11 && seconds <= 13) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0

    // lcd.print("       Pulses       ");
    lcd.print("   1 Second Clocks  ");

  }




  else if(seconds == 14) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0

    lcd.print("                    ");
  }


  else if(seconds > 14 && seconds <= 16) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0
    lcd.print(" Slow Pulses "); // miss pulse detected so extra 1 second motor pulse added
    lcd.print(misscount);
   }

 else if(seconds == 17) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0

    lcd.print("                    ");
  }


else if(seconds > 17 && seconds <= 21 && yearmiss != 00 ) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0
    
    //lcd.print(hourmiss);
    printLCDH(hourmiss); // adds leadng zero if hour is less than 10
   // lcd.print(":");
    //lcd.print(minutemiss);
   // lcd.print(":");
    printLCD(minutemiss); // adds leadng zero if minutes is less than 10
    //lcd.print(secondmiss);
    printLCD(secondmiss); // adds leadng zero if seconds is less than 10
    lcd.print("  ");
    
    
    printLCDH(daymiss); // adds leadng zero if day is less than 10
    lcd.print("/");
    printLCDH(monthmiss); // adds leadng zero if month is less than 10
    lcd.print("/");
    lcd.print("20");
    lcd.print(yearmiss);
    
    
  }
 else if(seconds > 17 && seconds <= 21 && yearmiss == 00 ) // if no fast pulses "Never" is printed not date
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0
    
    lcd.print("     Never     ");
   
    
  }

else if(seconds == 22) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0

    lcd.print("                    ");
  }
 else if(seconds > 22 && seconds <= 26) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0
    lcd.print("  Fast Pulses "); // miss pulse detected so extra 1 second motor pulse added
    lcd.print(extracount);
   }

 else if(seconds == 27) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0

    lcd.print("                    ");
  }


else if(seconds > 27 && seconds <= 31 && yearextra != 00 ) 
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0
    
     //lcd.print(hourmiss);
    printLCDH(hourextra); // adds leadng zero if hour is less than 10
   // lcd.print(":");
    //lcd.print(minutemiss);
   // lcd.print(":");
    printLCD(minuteextra); // adds leadng zero if minutes is less than 10
    //lcd.print(secondmiss);
    printLCD(secondextra); // adds leadng zero if seconds is less than 10
    lcd.print("  ");
    
    printLCDH(dayextra); // adds leadng zero if day is less than 10
    lcd.print("/");
    printLCDH(monthextra); // adds leadng zero if month is less than 10
    lcd.print("/");
    lcd.print("20");
    lcd.print(yearextra); // adds leading zero if year is less than 10
    
  }
  else if(seconds > 27 && seconds <= 31 && yearextra == 00 ) // if no fast pulses "Never" is printed not date
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0
    
    lcd.print("     Never     ");
   
    
  }
  

else if(seconds == 32) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0

    lcd.print("                    ");
  }

  else if(seconds > 32 && seconds <=59) // Ignored while clock is correcting forward or retarding
  {
    lcd.setCursor(0,1); //Start at character 0 on line 0
    lcd.print("  Slow "); // miss pulse detected so extra 1 second motor pulse added
    lcd.print(misscount);

    lcd.print(" ");
    lcd.print("   Fast ");        
    lcd.print(extracount);
  }



   firstRun = false;
  // End display of slow and fast pulses on row 01
  
  }
  

  
  void printLCD(int LCDdigits){
  // utility function for digital clock display: prints preceding colon and leading 0 on mins and secs
  lcd.print(":");
  if(LCDdigits < 10)
    lcd.print('0');
  lcd.print(LCDdigits); 

}




void printLCDH(int LCDdigitH){
  // utility function for digital clock display: prints leading 0 on hours
  if(LCDdigitH < 10)  
    lcd.print("0"); // Print hour on first line
  lcd.print(LCDdigitH);
}




void printPTen(int PTen){ // light LED every min on 00

  if(PTen == 00)
  {
    //Serial.print(PTen);
    digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)

  }
  else if(PTen != 00)

    digitalWrite(led, LOW);   // turn the LED off (LOW is the voltage level)

}





void printP30(int P30){ // light LED every 30 seconds

  if(P30 == 00 || P30 == 30)
  {
   // Serial.print(P30);
    digitalWrite(led30, HIGH);   // turn the LED on (HIGH is the voltage level)

  }
  else if(P30 != 00 || P30 !=30)

    digitalWrite(led30, LOW);   // turn the LED off (LOW is the voltage level)

  else if(P30 == 59 || P30 == 29)// makes sure clock pulses 30 sec pulse if DCF77 time corrects @ 59 or 29 sec

  {

   // Serial.print(P30);
    digitalWrite(led30, HIGH);   // turn the LED on (HIGH is the voltage level)
  }
  else if(P30 != 59 || P30 !=29) //// makes sure clock pulses turn off 30 sec pulse if DCF77 time corrects @ 59 or 29 sec

    digitalWrite(led30, LOW);   // turn the LED off (LOW is the voltage level)
 }




void printP05(int P05){ // light LED 10 every 5 seconds

  if(P05 == 00 || P05 == 05 || P05 == 10 || P05 == 15 || P05 == 20 || P05 == 25 || P05 == 30 || P05 == 35 || P05 == 40 || P05 == 45 || P05 == 50 || P05 == 55)
  {
   // Serial.print(P05);
    digitalWrite(led05, HIGH);   // turn the LED on (HIGH is the voltage level)

  }
  else if(P05 != 00 || P05 !=05 || P05 !=10 || P05 !=15 || P05 !=20 || P05 !=25 || P05 !=30 || P05 !=35 || P05 !=40 || P05 !=45 || P05 !=50 || P05 !=55)

    digitalWrite(led05, LOW);   // turn the LED off (LOW is the voltage level)
  }





void printP10(int P10){ // light LED 09 every 10 seconds

  if(P10 == 00 || P10 == 10 || P10 == 20 || P10 == 30 || P10 == 40 || P10 == 50)
  {
    //Serial.print(P10);
    digitalWrite(led10, HIGH);   // turn the LED on (HIGH is the voltage level)
     }
  else if(P10 != 00 || P10 !=10 || P10 !=20 || P10 !=30 || P10 !=40 || P10 !=50)

    digitalWrite(led10, LOW);   // turn the LED off (LOW is the voltage level)

}

/*
void printP15(int P15){ // light LED 03 every 15 seconds

  if(P15 == 00 || P15 == 15 || P15 == 30 || P15 == 45)
  {
  // Serial.print(P15);
    digitalWrite(led15, HIGH);   // turn the LED on (HIGH is the voltage level)
  }
  else if(P15 != 00 || P15 !=15 || P15 !=30 || P15 !=45)

    digitalWrite(led15, LOW);   // turn the LED off (LOW is the voltage level)

}
*/
void printP20(int P20){ // light LED 08 every 20 seconds

  if(P20 == 00 || P20 == 20 || P20 == 40)
  {
   //Serial.print(P20);
    digitalWrite(led20, HIGH);   // turn the LED on (HIGH is the voltage level)
  }
  else if(P20 != 00 || P20 !=20 || P20 !=40)

    digitalWrite(led20, LOW);   // turn the LED off (LOW is the voltage level)
}


void printPhour(int Phour){ // light LED every hour on 00

  if(Phour == 00 && seconds == 00)
  {
  // Serial.print(Phour);
    digitalWrite(ledh, HIGH);   // turn the LED on (HIGH is the voltage level)
  }
  else if(seconds != 00) // turn off hour pulse when secs are not 00

      digitalWrite(ledh, LOW);   // turn the LED off (LOW is the voltage level)

}
void printPday(int Pday){ // light LED every day on 00

  if(Pday == 00 && seconds == 00)
  {
    //Serial.print(Pday);
    digitalWrite(ledd, HIGH);   // turn the LED on (HIGH is the voltage level)
  }
  else if(seconds != 00) //turn off day pulse when secs are not 00

      digitalWrite(ledd, LOW);   // turn the LED off (LOW is the voltage level)


}


//================================================================================================================
//steps clock movement
 void secondsmotor (){
   
 /*  
  digitalWrite(quartzmotor1, coilStatus);
  digitalWrite(quartzmotor2, !coilStatus);
  coilStatus = !coilStatus;
} //end of <coilPulse> return to <loop>
*/

   // Quartz clock driver
  // toggle Quartz drive A2 & A3 evey second
  
  if (quartzmotor1 == LOW)
  { 
    quartzmotor1 = HIGH;
  }
  else
    quartzmotor1 = LOW;
  {
    digitalWrite(quartz01, quartzmotor1); // set the quartz motor drive A2 pin
  }
  if (quartzmotor2 == HIGH)
  { 
    quartzmotor2 = LOW;
  }
  else
    quartzmotor2 = HIGH;
  {
    digitalWrite(quartz02, quartzmotor2); // set the quartz motor drive A3 pin     

  }
  }



//================================================================================================================

// extraPulse
//
// called from <loop>
//================================================================================================================
//steps clock an extra pulse (uses A NOP command since delay does not work in this sketch)
void extraPulse(){
  //wait for approx 0.5 seconds (delay command seems to be disabled by the DCF library)
  for(int x = 0; x < 20 ; x++){
    for(int y = -32678; y < 32767; y++){
     NOP;
    }
  }
  /*
  digitalWrite(quartzmotor1, coilStatus);
  digitalWrite(quartzmotor2, !coilStatus);
  coilStatus = !coilStatus;
  */
   // toggle Quartz drive A2 & A3 evey second
  
  if (quartzmotor1 == LOW)
  { 
    quartzmotor1 = HIGH;
  }
  else
    quartzmotor1 = LOW;
  {
    digitalWrite(quartz01, quartzmotor1); // set the quartz motor drive A2 pin
  }
  if (quartzmotor2 == HIGH)
  { 
    quartzmotor2 = LOW;
  }
  else
    quartzmotor2 = HIGH;
  {
    digitalWrite(quartz02, quartzmotor2); // set the quartz motor drive A3 pin     

  }
  
} //end of <extraPulse> return to <loop>







void blankrow3() {
  lcd.setCursor(0,3);
  lcd.print("               "); 
}





void signalmatch() {

  signalQual = DCF77_Clock::get_prediction_match();
  if(signalQual == 255 || signalQual == 0 )
  {
    signalQual = 00;
  }
  else
  {
    signalQual = signalQual * 2;
   } 
   
    lcd.setCursor(0,3);
    lcd.print("Sig Match ");
    lcd.setCursor(13,3);
    lcd.print("%");
    
    if(signalQual > 99)
    { 
      lcd.setCursor(10,3);
    lcd.print(signalQual);
    }
    else if(signalQual > 9 && signalQual < 100 ) 
   {
   lcd.setCursor(10,3);
    lcd.print(" ");
    lcd.print(signalQual);
    
   }
    else if(signalQual < 10) 
   {
   lcd.setCursor(10,3);
    lcd.print("  ");
    lcd.print(signalQual);
   
    }
}
    /*
     lcd.setCursor(0,3);
  lcd.print("Sig Match ");
  lcd.print(signalQual);
  lcd.print("%");
}
*/


void precision() {
  lcd.setCursor(0,3);
  lcd.print("Accuracy ");
  lcd.print(DCF77_Frequency_Control::get_confirmed_precision());
  lcd.print("Hz "); 
}

/*
void freqadj() {
  lcd.setCursor(0,3);
 
 // lcd.print(16000000L - DCF77_1_Khz_Generator::read_adjustment());
  lcd.print(16000000L - Generic_1_kHz_Generator::read_adjustment());
  lcd.print("Hz"); 
}
*/






